part of ecs;

class EntityManager 
{
  
  int lastEntityId = 1;
  HashMap<Type, HashMap<int, EntityComponent>> componentStores = new HashMap<Type, HashMap<int, EntityComponent>>();
  
  // =============================
  // generates new entity id's
  createEntity() {
    return lastEntityId++;
  }
  
  // =============================
  // removes an entity from all component stores
  removeEntity(int entity) {
    componentStores.forEach((key, store) {
      if (store.containsKey(entity)) {
        store.remove(entity);
      }
    });
  }
  
  // =============================
  // retrieves a component for the given entity id
  HashMap<int, EntityComponent> getComponentStore(Type componentClass) {
    // get the component store
    HashMap<int, EntityComponent> store = componentStores[componentClass];
    return store;
  }
  
  // =============================
  // adds a component to an entity
  addComponent(int entity, Type componentClass, EntityComponent instance) {
    // get component store or create if absent
    HashMap<int, EntityComponent> store = componentStores.putIfAbsent(
        componentClass, 
        () => new HashMap<int, EntityComponent>() 
    );
    
    // add the component
    store[entity] = instance;
  }
  
  // =============================
  // retrieves a component for the given entity id
  EntityComponent getComponent(int entity, Type componentClass) {
    // get the component store
    HashMap<int, EntityComponent> store = componentStores[componentClass];
    
    // check the component store isn't null
    if (store == null)
        throw new Exception("There are no entities for this component.");
    
    return store[entity];
  }
  
  // =============================
  // removes the component type from the entity
  removeComponent(int entity, Type componentClass) {
    // get the component store
    HashMap<int, EntityComponent> store = componentStores[componentClass];
    
    // attempt to remove the entity
    store.remove(entity);
  }
  
}

// Component Type Def
abstract class EntityComponent {}