library ecs_tests;
import 'package:unittest/unittest.dart';
import 'package:ecs/ecs.dart';
part 'artifacts/test_components.dart';

void main() {
  EntityManager ecs;
  var testInstance1, testInstance2, testInstance3;
  
  // =============================
  setUp(() {
    
    ecs = new EntityManager();
    
  });
  
  addTestComponents(testEntity) {
    
    testInstance1 = new Component1(1.0,2.0,3.0);
    testInstance2 = new Component2(4.0,5.0,6.0);
    testInstance3 = new Component3(7.0,8.0,9.0);
    
    // add the component to the entity
    ecs.addComponent(testEntity, Component1, testInstance1);
    ecs.addComponent(testEntity, Component2, testInstance2);
    ecs.addComponent(testEntity, Component3, testInstance3);
    
  }
  
  // =============================
  test("Can create entity id's", () {
    
    // test
    
    int entityId1 = ecs.createEntity();
    int entityId2 = ecs.createEntity();
    int entityId3 = ecs.createEntity();
    
    // assert
    
    expect(entityId1, 1);
    expect(entityId2, 2);
    expect(entityId3, 3);
    
  });
  
  // =============================
  test("Can add and get components for an entity", () {
    
    // setup
    
    int testEntity = ecs.createEntity();
    
    // add the test components to the testEntity
    addTestComponents(testEntity);
    
    // test
    
    Component1 storedComponent1 = ecs.getComponent(testEntity, Component1);
    Component2 storedComponent2 = ecs.getComponent(testEntity, Component2);
    Component3 storedComponent3 = ecs.getComponent(testEntity, Component3);
    
    // assert
    
    // check object
    expect(storedComponent1, testInstance1);
    expect(storedComponent2, testInstance2);
    expect(storedComponent3, testInstance3);
    
    // check correct data is present
    expect(storedComponent1.x, testInstance1.x);
    expect(storedComponent1.y, testInstance1.y);
    expect(storedComponent1.z, testInstance1.z);
    
    expect(storedComponent2.dx, testInstance2.dx);
    expect(storedComponent2.dy, testInstance2.dy);
    expect(storedComponent2.dz, testInstance2.dz);
    
    expect(storedComponent3.vx, testInstance3.vx);
    expect(storedComponent3.vy, testInstance3.vy);
    expect(storedComponent3.vz, testInstance3.vz);
    
  });
  
  // =============================
  test("Can remove an entity", () {
    
    // setup
    
    // create our test entity
    int testEntity = ecs.createEntity();
    
    // add the test components to the testEntity
    addTestComponents(testEntity);
    
    // test
    
    // remove the entity
    ecs.removeEntity(testEntity);
    
    // assert
    
    // ensure the stored components are null
    Component1 storedComponent1 = ecs.getComponent(testEntity, Component1);
    Component2 storedComponent2 = ecs.getComponent(testEntity, Component2);
    Component3 storedComponent3 = ecs.getComponent(testEntity, Component3);
    
    expect(storedComponent1, null);
    expect(storedComponent2, null);
    expect(storedComponent3, null);
    
  });
  
  // =============================
  test("Can remove a component from an entity", () {
    
    // setup
    
    // create our test entity
    int testEntity = ecs.createEntity();
    
    // add the test components to the testEntity
    addTestComponents(testEntity);
    
    // test
    
    ecs.removeComponent(testEntity, Component1);
    ecs.removeComponent(testEntity, Component2);
    ecs.removeComponent(testEntity, Component3);
    
    // assert
    
    // ensure the stored components are null
    Component1 storedComponent1 = ecs.getComponent(testEntity, Component1);
    Component2 storedComponent2 = ecs.getComponent(testEntity, Component2);
    Component3 storedComponent3 = ecs.getComponent(testEntity, Component3);
    
    expect(storedComponent1, null);
    expect(storedComponent2, null);
    expect(storedComponent3, null);
    
  });
  
}